package com.gree.mall.contest.logic;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.json.JSONUtil;
import com.aliyuncs.utils.StringUtils;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.bean.Freight.FreightTemplateBean;
import com.gree.mall.contest.bean.Freight.RegionTreeBean;
import com.gree.mall.contest.bean.admin.AdminUserCom;
import com.gree.mall.contest.bean.order.BuyGood;
import com.gree.mall.contest.bean.order.OrderBuyBean;
import com.gree.mall.contest.bean.user.CurrentCompanyWechat;
import com.gree.mall.contest.enums.FreightTypeEnum;
import com.gree.mall.contest.enums.RegionLevelEnum;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.plus.entity.*;
import com.gree.mall.contest.plus.service.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
@RequiredArgsConstructor
public class FreightLogic {

    private final FreightTemplateService freightTemplateService;
    private final FreightTemplateDetailService freightTemplateDetailService;
    private final RegionService regionService;
    private final CommonLogic commonLogic;
    private final UserAddressService userAddressService;
    private final GoodsService goodsService;

    /**
     * 运费模板列表
     */
    public IPage<FreightTemplate> list(HttpServletRequest request, String name, Boolean status, String startTime, String endTime, Integer pageNo, Integer pageSize) {

        //获取当前登录企业微信id
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        List<String> companyWechatIds = adminUser.getCompanyWechatIds();

        return freightTemplateService.lambdaQuery()
                .in(CollectionUtil.isNotEmpty(companyWechatIds), FreightTemplate::getCompanyWechatId, companyWechatIds)
                .like(StringUtils.isNotEmpty(name), FreightTemplate::getName, name)
                .eq(status != null, FreightTemplate::getStatus, status)
                .between(StringUtils.isNotEmpty(startTime) && StringUtils.isNotEmpty(endTime), FreightTemplate::getCreateTime, startTime, endTime + " 23:59:59")
                .page(new Page<>(pageNo, pageSize));
    }

    /**
     * 运费模板详情
     */
    public FreightTemplateBean detail(String freightTemplateId) {
        FreightTemplate freightTemplate = freightTemplateService.getById(freightTemplateId);
        FreightTemplateBean bean = new FreightTemplateBean();
        BeanUtils.copyProperties(freightTemplate, bean);
        //查询列表
        List<FreightTemplateDetail> list = freightTemplateDetailService.lambdaQuery().eq(FreightTemplateDetail::getFreightTemplateId, freightTemplateId).list();
        bean.setFreightTemplateDetails(list);
        return bean;
    }

    /**
     * 删除运费模板
     */
    @Transactional
    public void delete(String freightTemplateId) {
        freightTemplateService.removeById(freightTemplateId);
        freightTemplateDetailService.lambdaUpdate().eq(FreightTemplateDetail::getFreightTemplateId, freightTemplateId).remove();
    }

    /**
     * 保存运费模板
     * 带freightTemplateId则为修改，否则新增
     */
    @Transactional
    public void save(FreightTemplateBean freightTemplateBean, HttpServletRequest request) {

        //设置企业微信id
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        if (StringUtils.isEmpty(adminUser.getLoginCompanyWechatId())) {
            throw new RemoteServiceException("无效微信企业id");
        }

        freightTemplateBean.setCompanyWechatId(adminUser.getLoginCompanyWechatId());
        freightTemplateBean.setCompanyName(adminUser.getLoginCompanyName());
        freightTemplateBean.setCreateUserName(adminUser.getNickName());
        freightTemplateBean.setCreateTime(new Date());
        freightTemplateService.saveOrUpdate(freightTemplateBean);

        if (freightTemplateBean.getFreightTemplateDetails() == null || freightTemplateBean.getFreightTemplateDetails().size() < 1) {
            return;
        }
        //删除运费明细
        if (StringUtils.isNotEmpty(freightTemplateBean.getFreightTemplateId())) {
            freightTemplateDetailService.lambdaUpdate()
                    .eq(FreightTemplateDetail::getFreightTemplateId, freightTemplateBean.getFreightTemplateId()).remove();
        }
        //新增运费明细
        List<FreightTemplateDetail> freightTemplateDetails = freightTemplateBean.getFreightTemplateDetails();
        for (FreightTemplateDetail freightTemplateDetail : freightTemplateDetails) {
            freightTemplateDetail.setFreightTemplateId(freightTemplateBean.getFreightTemplateId());
            freightTemplateDetail.setCompanyWechatId(adminUser.getLoginCompanyWechatId());
            freightTemplateDetail.setCompanyName(adminUser.getLoginCompanyName());
            freightTemplateDetail.setCreateTime(new Date());
        }
        freightTemplateDetailService.saveBatch(freightTemplateDetails);
    }

    /**
     * 启用/禁用
     */
    @Transactional
    public void updateStatus(String freightTemplateId, Boolean status) {

        freightTemplateService.lambdaUpdate().set(FreightTemplate::getStatus, status)
                .eq(FreightTemplate::getFreightTemplateId, freightTemplateId).update();

        freightTemplateDetailService.lambdaUpdate()
                .set(FreightTemplateDetail::getStatus, status)
                .eq(FreightTemplateDetail::getFreightTemplateId, freightTemplateId)
                .update();
    }


    /**
     * 根据件数以及地区计算运费
     */
    public BigDecimal getFreightAmount(HttpServletRequest request, String province, String city, String area, Integer num) throws RemoteServiceException {

        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        List<String> companyWechatIds = adminUser.getCompanyWechatIds();

        FreightTemplateDetail one = freightTemplateDetailService.lambdaQuery()
                .in(CollectionUtil.isNotEmpty(companyWechatIds), FreightTemplateDetail::getCompanyWechatId, companyWechatIds)
                .eq(FreightTemplateDetail::getProvince, province)
                .eq(FreightTemplateDetail::getCity, city)
//                .eq(FreightTemplateDetail::getArea, area)
                .eq(FreightTemplateDetail::getStatus, true).one();
        if (one == null) {
            throw new RemoteServiceException("暂不支持该地区的配送服务");
        }
        Integer firstLimit = one.getFirstLimit();
        Integer continuousLimit = one.getContinuousLimit();
        if (num <= firstLimit) {
            return one.getFirstAmount();
        }
        //续件费用
        int xj = (num - firstLimit) / continuousLimit + ((num - firstLimit) % continuousLimit == 0 ? 0 : 1);
        BigDecimal conAmount = one.getContinuousAmount().multiply(new BigDecimal(xj));

        return one.getFirstAmount().add(conAmount);
    }


    /**
     * 获取地区树
     *
     * @return
     */
    public List<RegionTreeBean> regionTree(Integer level) {
        if (level == null) {
            level = 3;
        }
        List<Region> regions = regionService.lambdaQuery()
                .le(Region::getLevel, level)
                .list();

        List<RegionTreeBean> list = new ArrayList<>();
        for (int i = 0; i < regions.size(); i++) {
            RegionTreeBean regionTree = new RegionTreeBean();
            BeanUtils.copyProperties(regions.get(i), regionTree);
            list.add(regionTree);
        }
        //省
        List<RegionTreeBean> province = list.stream().filter(v -> v.getLevel().equals(RegionLevelEnum.PROVINCE.getId())).collect(Collectors.toList());
        for (RegionTreeBean regionTree : province) {
            regionTree.setChild(this.regionTree(list, regionTree.getId(), regionTree.getName()));
        }
        return province;
    }

    private List<RegionTreeBean> regionTree(List<RegionTreeBean> list, Long parentId, String parentName) {
        if (CollectionUtil.isEmpty(list)) {
            return new ArrayList<>();
        }
        List<RegionTreeBean> childs = list.stream().filter(v -> v.getPid().equals(parentId)).collect(Collectors.toList());
        for (RegionTreeBean regionTree : childs) {
            regionTree.setParentName(parentName);
            regionTree.setChild(this.regionTree(list, regionTree.getId(), regionTree.getName()));
        }
        return childs;
    }

    public BigDecimal getFreightAmountV2(OrderBuyBean freightBean, HttpServletRequest request) throws RemoteServiceException {
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
        if (StringUtils.isNotEmpty(freightBean.getUserAddressId())) {
            UserAddress userAddress = userAddressService.getById(freightBean.getUserAddressId());
            BigDecimal freightAmountV2 = this.getFreightAmountV2(userAddress.getProvince(), userAddress.getCity(), userAddress.getArea(), freightBean.getBuyGoods(), currentCompanyWechat);
            return freightAmountV2;
        }
        return BigDecimal.valueOf(0);
    }

    public BigDecimal getFreightAmountV2(String province, String city, String area, List<BuyGood> buyGoods, AdminCompanyWechat adminCompanyWechat) throws RemoteServiceException {
        BigDecimal totalAmount = new BigDecimal(0.00);
        log.info("运费商品列表{}", JSONUtil.toJsonStr(buyGoods));
        if (buyGoods != null && buyGoods.size() > 0) {

            for (BuyGood buyGood : buyGoods) {
                BigDecimal freightItem = computeAmountItem(province, city, area, buyGood, adminCompanyWechat);
                totalAmount = totalAmount.add(freightItem);
            }
        }
        return totalAmount;
    }

    public BigDecimal computeAmountItem(String province, String city, String area, BuyGood buyGood, AdminCompanyWechat adminCompanyWechat) throws RemoteServiceException {
        Goods goods = goodsService.getById(buyGood.getGoodsId());
        if (goods != null && FreightTypeEnum.UNIFIED.toString().equals(goods.getFreightType())) {
            return goods.getFreight().multiply(new BigDecimal(buyGood.getNum()));
        }
        FreightTemplate freightTemplate = freightTemplateService.lambdaQuery()
                .eq(FreightTemplate::getFreightTemplateId, goods.getFreightTemplateId())
                .one();

        if (freightTemplate == null) {
            log.info("异常商品id{}", buyGood.getGoodsId());
            throw new RemoteServiceException("暂不支持该地区的配送服务...");
        }
        if (StringUtils.isNotEmpty(province) && StringUtils.isNotEmpty(city)) {
            FreightTemplateDetail one = freightTemplateDetailService.lambdaQuery()
                    .eq(FreightTemplateDetail::getFreightTemplateId, freightTemplate.getFreightTemplateId())
                    .eq(FreightTemplateDetail::getProvince, province)
                    .eq(FreightTemplateDetail::getCity, city)
//                .eq(FreightTemplateDetail::getArea, area)
                    .eq(FreightTemplateDetail::getCompanyWechatId, adminCompanyWechat.getCompanyWechatId())
                    .eq(FreightTemplateDetail::getStatus, true)
                    .one();
            if (one == null) {
                throw new RemoteServiceException("暂不支持该地区的配送服务");
            }
            Integer firstLimit = one.getFirstLimit();
            Integer continuousLimit = one.getContinuousLimit();
            int num = buyGood.getNum();
            BigDecimal conAmount = new BigDecimal(0.00);
            if (num > firstLimit) {
                int xj = (num - firstLimit) / continuousLimit + ((num - firstLimit) % continuousLimit == 0 ? 0 : 1);
                conAmount = one.getContinuousAmount().multiply(new BigDecimal(xj));
            }
            return one.getFirstAmount().add(conAmount);
        }
        return BigDecimal.valueOf(0);
    }
}
